/*############*/
/*# Includes #*/
/*############*/

#include <stdio.h>
#include <stdlib.h>

#ifdef   _UNIX_
#include <unistd.h>
#else
#define  R_OK  0
#endif

#include <string.h>
#include <errno.h>

#include "slidstat.h"

/*###########*/
/*# Defines #*/
/*###########*/

#define LIMIT  (nNeighbors/2)

/*######################*/
/*# External variables #*/
/*######################*/

extern char   verbose;
extern int    nProcs;
extern char   nNeighbors;
extern char   pYN[];

/*######################*/
/*# External functions #*/
/*######################*/

#ifdef _ANSI_
extern int  (*neighbor_id)(int iProc, char iNeighbor);
extern int  (*backlink_id)(int iProc, char iNeighbor);
#else
extern int  (*neighbor_id)();
extern int  (*backlink_id)();
#endif

/*####################*/
/*# Module variables #*/
/*####################*/

static int limit;

/*#############*/
/*# Functions #*/
/*#############*/

/*:::::::::::::::::*/
/*:: diag_init() ::*/
/*:::::::::::::::::*/

static char *MM = "--";

int diag_init(peDiag)
char *peDiag;
{
    FILE *pYF;

    if (*peDiag  == '\0')
	limit = (int)LIMIT;
    else if (sscanf(peDiag, "l%d", &limit) < 1 || limit > nNeighbors) {
	fprintf(stderr, "diag_init(): invalid parameter format!\n");
	exit(ERROR);
    }

    /* display parameters */
    printf("DIAGNOSE - SLIDSTAT diagnostic module\n");
    printf("--------   Version 1.0\n\n");
    if (strlen(peDiag) > 0)
        printf("Extra parameters for diagnostics '%s'.\n\n", peDiag);
    if ((verbose & SLS_SYS)  != 0) {
	if ((pYF = fopen(pYN, "a")) == NULL) {
	    fprintf(stderr, "diag_init(): file open error [%s]!\n", pYN);
	    exit(ERROR);
	}
	fprintf(pYF, ";dextra\n");
	fprintf(pYF, "%14s\n\n", (strlen(peDiag) > 0) ? peDiag : MM);
	fclose (pYF);
    }
    return (OK);
}

/*:::::::::::::::::*/
/*:: diagnosis() ::*/
/*:::::::::::::::::*/
int diagnosis(psArray, iRound)
PROCSTAT *psArray;
int       iRound;
{
    int  i, j;
    int  iTester;
    char fSum;

    for (i = 0; i < nProcs; i++) {
	for (j = fSum = 0; j < nNeighbors; j++) {
	    if ((iTester = (*backlink_id)(i, j)) == NONEXIST)
		continue;
	    fSum += ((psArray + iTester)->tests[j] == FAULTY) ? 1 : 0;
	}
	(psArray + i)->diag = (fSum > (char)limit) ? FAULTY : FAULT_FREE;
/*
	if ((psArray + i)->diag != (psArray + i)->state) {
	    printf("[%2x]", i);
	    for (j = fSum = 0; j < nNeighbors; j++) {
		if ((iTester = (*backlink_id)(i, j)) == NONEXIST)
		    continue;
		printf("<%2x:%d>", iTester, ((psArray + iTester)->tests[j] == FAULTY) ? 1 : 0);
	    }
	    printf(" ");
	}
 */
    }

    return (OK);
}

/*::::::::::::::::::*/
/*:: diag_close() ::*/
/*::::::::::::::::::*/
void diag_close()
{
}
