/* $Id$ */

#ifndef __slidstat_h
#define __slidstat_h

/*#############*/
/*# Constants #*/
/*#############*/

#define FALSE  0
#define TRUE   (!FALSE)
#define OK     0
#define ERROR  (-1)

#ifdef  _UNIX_
#define MAX_ROUND     65534							/* Maximal number of evaluation rounds     */
#else
#define MAX_ROUND     32766
#endif
#define MAX_PROC      32766							/* Maximal dimensions: 32*32*32 processors */
#define MAX_NEIGHBOR      8							/* Maximal number of neighbors (hypercube) */
#define MAX_FAULT     32766							/* Maximal number of faulty processors     */
#define MAX_FGROUP     8094							/* Maximal number of fault groups          */
#define NUM_INVAL         9							/* Number of test invalidation models      */
#define MIN_IBLOCK        2							/* Minimal number of invalidation blocks   */
#define NUM_TOPOLOGY     10							/* Number of implemented system topologies */
#define NUM_INJECTOR      8							/* Number of implemented injection methods */
#define NUM_COEFF         5							/* Number of confidence coefficients       */
#define NUM_SAMPLE        9							/* Number of sample sizes                  */
#define MAX_SAMPLE     2000							/* Maximal number of samples               */

#define NUM_STRING      254

#define FAULT_FREE  0x01
#define FAULTY      0x02
#define UNKNOWN     0x04							/* Unknown fault state         */
#define NONEXIST    (-1)							/* Nonexistent processor index */

#define TI_00  0x01								/* Ivalidation: 0-fail-safe tester */
#define TI_01  0x02								/*              perfect tester     */
#define TI_0X  0x03								/*              sequential tester  */
#define TI_10  0x04								/*              consistent liar    */
#define TI_11  0x05								/*              irreflexive, HK2   */
#define TI_1X  0x06								/*              reflexive, HK1     */
#define TI_X0  0x07								/*              imperfect traitor  */
#define TI_X1  0x08								/*              asymmetric, BGM    */
#define TI_XX  0x00								/*              symmetric, PMC     */

#define SLS_SPEC  0x01                                                          /* Status of specification file          */
#define SLS_SYS   0x02                                                          /*           system-wide statistics file */
#define SLS_SEL   0x04                                                          /*           selective statistics file   */
#define SLS_LOG   0x08                                                          /*           diagnostic log file         */
#define SLS_DIAG  0x10                                                          /*           verbose information file    */

#define DL_CFT2   0x01111
#define DS_CIP1   0x02111
#define DS_CIP2   0x02211
#define DS_LMIM1  0x03112
#define DS_LMIM2  0x03112
#define DS_DIL1   0x04111
#define DS_LTC1   0x05111
#define DS_LTC2   0x05111

/*###########*/
/*# Defines #*/
/*###########*/

#ifdef  _UNIX_
#define min(a,b)      ((a) < (b) ? (a) : (b))
#define max(a,b)      ((a) > (b) ? (a) : (b))
#endif
#define sqr(x)        (x)*(x)
#define cube(x)       (x)*(x)*(x)
#define root(x,n)     exp(log(x)/(n))
#define percent(x,n)  (int)ceil((double)(x)/(double)(n)*100.0)

/*############*/
/*# Typedefs #*/
/*############*/

typedef char byte;

typedef struct procstat {
    char state;									/* Processor: fault state                   */
    char inval;									/*            test invalidation type        */
    char tests[MAX_NEIGHBOR];							/*            results of tests on neighbors */
    char diag;									/*            diagnosed state               */
    int  benign;								/* Number of misdiagnoses: GOOD -> FAULTY   */
    int  malign;								/*                         FAULTY -> GOOD   */
    int  unknown;								/* Number of UNKNOWN classifications        */
} PROCSTAT;
#define PSTATSIZE sizeof(PROCSTAT)

typedef struct proctab {
    int  nProcs;								/* Number of processors            */
    int  nGroups;								/*           processor groups      */
    int *prList;								/*   List of processor identifiers */
} PROCTAB;
#define PTABSIZE sizeof(PROCTAB)

typedef struct mdentry {
    int             iProc;							/* Misdiagnosis: processor identifier */
    char            mdType;							/*               type                 */
    struct mdentry *pNext;							/*               next list item       */
} MDENTRY;
#define MDESIZE  sizeof(MDENTRY)

typedef struct stround {
    int benign;									/* Number of benign misdiagnosed rounds */
    int malign;									/*           malign misdiagnosed rounds */
    int mixed;									/*           rounds with both types     */
    int unknown;								/*           rounds with incompleteness */
} STROUND;
#define STRSIZE  sizeof(STROUND)

typedef struct stclass {
    int    cur;									/*  Actual number of incorrect diagnoses  */
    long   tot;									/*   Total number of                      */
    int    min;
    int    max;									/* Maximal number of                      */
    double dvn;									/*       Variance of                      */
    float  itv;									/* confidence interval for the mean       */
} STCLASS;

typedef struct stdata {
    STROUND   r;								/* Round     statistics           */
    STROUND *pr;								/* Processor statistics           */
    STCLASS  mg;								/* Misdiagnosed GOOD   processors */
    STCLASS  mf;								/*              FAULTY processors */
    STCLASS   u;								/*        Unidentified processors */
} STDATA;

struct stats {
    int      rounds;								/* Number of completed rounds              */
    int      repeats;								/*           rounds with repeated patterns */
    STCLASS   g;								/* Number of GOOD   processors             */
    int     *pg;
    STCLASS   f;								/*           FAULTY processors             */
    int     *pf;
    STDATA   sys;								/* System-wide statistics data             */
    STDATA   sel;								/* Selective statistics data               */
};

#endif
